@@ -33,15 +33,6 @@
 #define INDEX_FILESYSTEM    3
 #define INDEX_DOCUMENTS     4
 
-#define NAME_SD_CARD        "SD-MMC Card"
-#define NAME_ROM_STORAGE    "ROM Storage"
-
-enum {
-  FS_DEVICE = 0,
-  FS_SD_CARD,
-  FS_ROM_STORAGE
-};
-
 /* structures for info held by gnomevfs between calls */
 
 typedef struct _VFS_DIR_HANDLE
@@ -74,16 +65,12 @@ typedef struct _ErrorCodeTriple
   HRESULT hresult;
 } ErrorCodeTriple;
 
-/* hack, these should be in synce.h */
-#ifndef ERROR_NOT_SAME_DEVICE
-#define ERROR_NOT_SAME_DEVICE         17
-#endif
-
 static ErrorCodeTriple error_codes[] =
 {
     {GNOME_VFS_OK,                        ERROR_SUCCESS,        S_OK        },
     {GNOME_VFS_ERROR_GENERIC,             0,                    E_FAIL      },
     {GNOME_VFS_ERROR_INTERNAL,            0,                    E_UNEXPECTED},
+    {GNOME_VFS_ERROR_NOT_SUPPORTED,       0,                    E_NOTIMPL},
     {GNOME_VFS_ERROR_NOT_FOUND,           ERROR_FILE_NOT_FOUND, 0},
     {GNOME_VFS_ERROR_NOT_FOUND,           ERROR_PATH_NOT_FOUND, 0},
     {GNOME_VFS_ERROR_INVALID_URI,         ERROR_INVALID_NAME,   0},
@@ -1617,7 +1604,13 @@ static GnomeVFSResult synce_same_fs/*{{{*/
 {
   GnomeVFSResult result;
   gchar *location_a = NULL, *location_b = NULL;
-  gint fs_a, fs_b, index_a, index_b;
+  gint index_a, index_b;
+  gboolean fs_a_device, fs_b_device;
+  DWORD attributes;
+  LPWSTR wide_root_dir = NULL;
+  gchar *fs_a_root = NULL;
+  gchar *fs_b_root = NULL;
+  RapiConnection *rapi_conn = NULL;
 
   synce_debug("%s: ------ entering ------", G_STRFUNC);
 
@@ -1645,42 +1638,70 @@ static GnomeVFSResult synce_same_fs/*{{{*/
   }
 #endif
 
-  fs_a = FS_DEVICE;
+  MUTEX_LOCK (mutex);
+  if ((result = initialize_rapi(a, &rapi_conn)) != GNOME_VFS_OK) {
+    MUTEX_UNLOCK (mutex);
+    goto exit;
+  }
+
+  fs_a_device = TRUE;
   if (index_a == INDEX_FILESYSTEM) {
     gchar **split = g_strsplit(location_a, "\\", 0);
 
     if (split && split[0] && split[1]) {
-      if (strcmp(split[1], NAME_SD_CARD) == 0)
-	fs_a = FS_SD_CARD;
-    }
-    if (split && split[0] && split[1]) {
-      if (strcmp(split[1], NAME_ROM_STORAGE) == 0)
-	fs_a = FS_ROM_STORAGE;
+
+            fs_a_root = g_strdup_printf("\\%s", split[1]);
+            wide_root_dir = wstr_from_utf8(fs_a_root);
+            attributes = CeGetFileAttributes(wide_root_dir);
+            wstr_free_string(wide_root_dir);
+
+            if ((attributes != 0xffffffff) && (attributes & FILE_ATTRIBUTE_TEMPORARY))
+                    /* get size for this */
+                    fs_a_device = FALSE;
+            else
+                    g_free(fs_a_root);
     }
     g_strfreev(split);
   }
 
-  fs_b = FS_DEVICE;
+  fs_b_device = TRUE;
   if (index_b == INDEX_FILESYSTEM) {
     gchar **split = g_strsplit(location_b, "\\", 0);
 
     if (split && split[0] && split[1]) {
-      if (strcmp(split[1], NAME_SD_CARD) == 0)
-	fs_b = FS_SD_CARD;
-    }
 
-    if (split && split[0] && split[1]) {
-      if (strcmp(split[1], NAME_ROM_STORAGE) == 0)
-	fs_b = FS_ROM_STORAGE;
-    }
+            fs_b_root = g_strdup_printf("\\%s", split[1]);
+            wide_root_dir = wstr_from_utf8(fs_b_root);
+            attributes = CeGetFileAttributes(wide_root_dir);
+            wstr_free_string(wide_root_dir);
 
+            if ((attributes != 0xffffffff) && (attributes & FILE_ATTRIBUTE_TEMPORARY))
+                    /* get size for this */
+                    fs_b_device = FALSE;
+            else
+                    g_free(fs_b_root);
+    }
     g_strfreev(split);
   }
 
-  if (fs_a == fs_b)
-    *same_fs_return = TRUE;
+  CeRapiUninit();
+  rapi_connection_destroy(rapi_conn);
+  MUTEX_UNLOCK (mutex);
+
+  if (fs_a_device && fs_b_device)
+          *same_fs_return = TRUE;
+  else if (fs_a_device != fs_b_device)
+          *same_fs_return = FALSE;
   else
-    *same_fs_return = FALSE;
+          {
+                  if (strcmp(fs_a_root, fs_b_root) == 0)          
+                          *same_fs_return = TRUE;
+                  else
+                          *same_fs_return = FALSE;
+          }
+
+  g_free(fs_a_root);
+  g_free(fs_b_root);
 
   result = GNOME_VFS_OK;
 
@@ -1705,6 +1726,10 @@ synce_get_volume_free_space
   gchar *location = NULL;
   gint index;
   RapiConnection *rapi_conn = NULL;
+  DWORD attributes;
+  gchar *root_dir = NULL;
+  LPWSTR wide_root_dir = NULL;
+  ULARGE_INTEGER FreeBytesAvailable, TotalNumberOfBytes, TotalNumberOfFreeBytes;
 
   synce_debug("%s: ------ entering ------", G_STRFUNC);
 
@@ -1720,30 +1745,45 @@ synce_get_volume_free_space
   }
 #endif
 
-  if (index == INDEX_FILESYSTEM) {
-    gchar **split = g_strsplit(location, "\\", 0);
-
-    if (split && split[0] && split[1]) {
-      if (strcmp(split[1], NAME_SD_CARD) == 0) {
-	result = GNOME_VFS_ERROR_NOT_SUPPORTED;
-	goto exit;
-      }
-
-      if (strcmp(split[1], NAME_ROM_STORAGE) == 0) {
-	result = GNOME_VFS_ERROR_NOT_SUPPORTED;
-	goto exit;
-      }
-
-    }
-    g_strfreev(split);
-  }
-
   MUTEX_LOCK (mutex);
   if ((result = initialize_rapi(uri, &rapi_conn)) != GNOME_VFS_OK) {
     MUTEX_UNLOCK (mutex);
     goto exit;
   }
 
+  if (index == INDEX_FILESYSTEM) {
+    gchar **split = g_strsplit(location, "\\", 0);
+
+    if (split && split[0] && split[1]) {
+            root_dir = g_strdup_printf("\\%s", split[1]);
+            wide_root_dir = wstr_from_utf8(root_dir);
+            attributes = CeGetFileAttributes(wide_root_dir);
+            wstr_free_string(wide_root_dir);
+
+            if ((attributes != 0xffffffff) && (attributes & FILE_ATTRIBUTE_TEMPORARY)) {
+                    if (CeGetDiskFreeSpaceEx(root_dir,
+                                             &FreeBytesAvailable,
+                                             &TotalNumberOfBytes,
+                                             &TotalNumberOfFreeBytes) != 0) {
+
+                            *free_space = TotalNumberOfFreeBytes;
+                            result = GNOME_VFS_OK;
+                    } else {
+                            synce_error("%s: Failed to get store information", G_STRFUNC);
+                            result = gnome_vfs_result_from_rapi(NULL);
+                    }
+
+                    g_free(root_dir);
+                    CeRapiUninit();
+                    rapi_connection_destroy(rapi_conn);
+                    MUTEX_UNLOCK (mutex);
+                    goto exit;
+            }
+            g_free(root_dir);
+    }
+    g_strfreev(split);
+  }
+
   if (CeGetStoreInformation(&store)) {
     *free_space = store.dwFreeSize;
     result = GNOME_VFS_OK;
